import 'package:contentsquare/src/modules/screengraph/screengraph_explorer.dart';
import 'package:contentsquare/src/modules/screengraph/screengraph_node.dart';
import 'package:flutter/widgets.dart';

/// ElementLocator is responsible for retrieving from the element tree all leaf elements which location matches the given position
class ElementLocator {
  static const _targetableWidgets = [
    'TextButton',
    'ElevatedButton',
    'FloatingActionButton',
    'IconButton',
    'InkWell',
    'GestureDetector',
    'DropdownButton',
    'OutlinedButton',
    'Radio',
    'PopupMenuButton',
  ];

  /// Scan the element tree for candidate elements at the given position
  static String scanRootForCandidateElementsAtPosition(Offset position) {
    final List<String> targetedElementsPaths = [];
    ScreengraphExplorer().searchForNodeAtPosition(position,
        (ScreengraphNode node) {
      final element = node.element;

      if (element.toDiagnosticsNode().getChildren().isEmpty ||
          _isWidgetTargetable(element)) {
        targetedElementsPaths.add(node.path);
        return;
      }
    });
    return targetedElementsPaths.last;
  }

  static bool _isWidgetTargetable(Element element) {
    return _targetableWidgets.contains(element.widget.runtimeType.toString());
  }
}
